#include "triangle.h"
#include "linesegment.h"

Triangle::Triangle(V3 v0, V3 v1, V3 v2) {

  verts[0] = v0;
  verts[1] = v1;
  verts[2] = v2;

}

void Triangle::DrawWireframe(FrameBuffer *fb, unsigned int color) {

  for (int i = 0; i < 3; i++) {
    V3 V0 = verts[i];
    V3 V1 = verts[(i+1)%3];
    LineSegment ls(V0, V1);
    ls.Draw2D(fb, color);
  }

}

void Triangle::Rasterize(FrameBuffer *fb, unsigned int color) {

  float leftf = verts[0][0];
  float rightf = verts[0][0];
  float topf = verts[0][1];
  float bottomf = verts[0][1];

  for (int i = 1; i < 3; i++) {
    if (leftf > verts[i][0])
      leftf = verts[i][0];
    if (rightf < verts[i][0])
      rightf = verts[i][0];
    if (topf > verts[i][1])
      topf = verts[i][1];
    if (bottomf < verts[i][1])
      bottomf = verts[i][1];
  }

  V3 edges[3];
  for (int i = 0; i < 3; i++) {
    int _i = (i+1)%3;
    edges[i][0] = verts[_i][1]-verts[i][1];
    edges[i][1] = verts[i][0]-verts[_i][0];
    edges[i][2] = -verts[i][0]*edges[i][0]-
      verts[i][1]*edges[i][1];
    if (edges[i]*verts[(i+2)%3] < 0)
      edges[i] = edges[i]*-1.0f;
  }

  for (int v = (int) topf; v <= (int) bottomf; v++) {
    for (int u = (int) leftf; u <= (int) rightf; u++) {
      V3 uvf(.5f+(float)u, .5f+(float)v, 1.0f);
      if (uvf*edges[0] < 0.0f 
        || uvf*edges[1] < 0.0f || 
        uvf*edges[2] < 0.0f)
        continue;
      fb->SetGuarded(uvf[0], uvf[1], color);
    }
  }

}
